<?php  
/* Plugin Name: Top Commentators Widget  
Description: Adds a sidebar widget to show the top commentators in your WP site. Adapted from Show Top Commentators plugin.  
Author: Lorna Timbah  
Version: 0.999 
Author URI: http://webgrrrl.net 
Plugin URI: http://webgrrrl.net/archives/my-top-commentators-widget-quick-dirty.htm
*/    

// Put functions into one big function we'll call at the plugins_loaded  
// action. This ensures that all required plugin functions are defined.

function widget_topcomm_init() {
	// Check for the required plugin functions. This will prevent fatal
  	// errors occurring when you deactivate the dynamic-sidebar plugin.
  	if ( !function_exists('register_sidebar_widget') )
  		return;
   	// This is the function that outputs our top commentators list
  	function widget_topcomm($args) {
 		// $args is an array of strings that help widgets to conform to
  		// the active theme: before_widget, before_title, after_widget,
  		// and after_title are the array keys. Default tags: li and h2.
  		extract($args);
   		// This is the function that truncates long commentators names to avoid
  		// your site design from breaking
  		function ns_substr_ellipse($str, $len) {
  			if(strlen($str) > $len) {
  				$str = substr($str, 0, $len-3) . "...";
  			}
  			return $str;
  		}
   		// This one string determines whether you want it to appear in the main page or everywhere
  		$options = get_option('widget_topcomm');
  		$showInHome = $options['showInHome'];
  		$title = htmlspecialchars(stripcslashes($options['title']), ENT_QUOTES);
  		global $wpdb;
  		if($showInHome == 1) {
  			if(is_home())
   				echo $before_widget . $before_title . $title . $after_title;
  		} else {
  			echo $before_widget . $before_title . $title . $after_title;
  		}
   		// This is the function that queries whether the commenters have entered a URL
  		function ns_get_user_url($user) {
  			global $wpdb;
  			$url = $wpdb->get_var("
  				SELECT comment_author_url, COUNT(comment_author_url) AS comment_aut_urlcount
 	 				FROM $wpdb->comments
  				WHERE comment_author = '".addslashes($user)."'
  				$filterUrls
  				GROUP BY comment_author_url
  				ORDER BY comment_aut_urlcount DESC LIMIT 1
  			");
  			return $url;
  		}
		function ns_show_top_commentators() {
			// Each widget can store its own options. We keep strings here.
			$options = get_option('widget_topcomm');

			// This prepares names for filtering
			if($options['excludeNames'] != "") {
				$excludeNames = trim($options['excludeNames']);
				$excludeNames = explode(",", $excludeNames);
				for($i=0; $i<count($excludeNames); $i++) {
					$new_names .= " AND comment_author NOT IN ('" . trim($excludeNames[$i]) . "')";
				}
				$excludeNames = $new_names;
			}

			if($options['limitList'] != "")
				$limitList = "LIMIT " . $options['limitList'];
			if($options['limitChar'] != "") {
				$limitChar = $options['limitChar'];
			} else {
				$limitChar = 20;
			}
			$listDesc = $options['listDesc'];
			$listType = $options['listType'];
			$listPeriod = $options['listPeriod'];
			if($options['listNull'] == "") {
				$listNull = "No commentators.";
			} else {
				$listNull = $options['listNull'];
			}
			$filterUrl = $options['filterUrl'];
			$makeLink = $options['makeLink'];
			// This sets the type of list to be used
			if($listType == "num") {
				$listStart = "<ol>";
				$listEnd = "</ol>";
			} else {
				$listStart = "<ul>";
				$listEnd = "</ul>";
			}
			// This prepares URLs for filtering
			$filterUrl = trim($filterUrl);
			$filterUrl = explode(",", $filterUrl);
			for($i=0; $i<count($filterUrl); $i++) {
				$new_urls .= " AND comments_author_url NOT LIKE '%" . trim($filterUrl[$i]) . "%'";
			}
			$filterUrl = $new_urls;
			// This is the function that prepare time period limitation for filtering
			if($listPeriod == "h") {
				$listPeriod = "DATE_FORMAT(comment_date, '$%Y-%m-%d %H') = DATE_FORMAT(CURDATE(), '%Y-%m-%d %H')";
			} elseif($listPeriod == "d") {
				$listPeriod = "DATE_FORMAT(comment_date, '%Y-%m-%d') = DATE_FORMAT(CURDATE(), '%Y-%m-%d')";
			} elseif($listPeriod == "w") {
				$listPeriod = "DATE_FORMAT(comment_date, '%Y-%v') = DATE_FORMAT(CURDATE(), '%Y-%v')";
			} elseif($listPeriod == "y") {
				$listPeriod = "DATE_FORMAT(comment_date, '%Y') = DATE_FORMAT(CURDATE(), '%Y')";
			} elseif($listPeriod == "a") {
				$listPeriod = "1=1";
			} elseif(is_numeric($listPeriod)) {
				$listPeriod = "comment_date >= CURDATE() - INTERVAL $listPeriod DAY";
			} else {
				$listPeriod = "DATE_FORMAT(comment_date, '%Y-%m') = DATE_FORMAT(CURDATE(), '%Y-%m')";
			}
			// These lines generate our output. Widgets can be very complex
			// but as you can see here, they can also be very, very simple.
			global $wpdb;
			$commenters = $wpdb->get_results("
				SELECT COUNT(comment_author) AS comment_comments, comment_author
				FROM $wpdb->comments
				WHERE comment_type != 'pingback'
				AND comment_author != ''
				AND comment_approved = '1'
				AND $listPeriod
				$excludeNames
				GROUP BY comment_author
				ORDER BY comment_comments DESC, comment_author ASC
				$limitList
				");
			if(is_array($commenters)) {
				$countList = 0;
				echo $listDesc . "\n";
				echo $listStart . "\n";
				foreach ($commenters as $k) {
					$url = ns_get_user_url($k->comment_author);
					echo "<li>";
					if(trim($url) != '' ) {
						if($makeLink == 1) {
							echo "<a href='" . $url . "'>";
						}
					}
					echo ns_substr_ellipse($k->comment_author, $limitChar);
					echo " (" . $k->comment_comments . ")";
					if(trim($url) != '') {
						if($makeLink == 1)
							echo "</a>";
					}
					echo "</li>\n";
					unset($url);
					$countList = $ $countList + 1;
				}
				if($countList == 0)
					echo "<li>" . $listNull . "</li>\n";
				echo $listEnd;
			} else {
				echo "<ul><li>" . $listNull . "</li></ul>" . "\n";
			}
		}
		if($showInHome == 1) {
			if(is_home())
				echo ns_show_top_commentators();
	  			echo $after_widget . "<!-- end of widget -->";
		} else {
			echo ns_show_top_commentators();
  			echo $after_widget . "<!-- end of widget -->";
		}
	}

	function widget_topcomm_control() {
		// Get our options and see if we're handling a form submission.
		$options = get_option('widget_topcomm');
		if ( !is_array($options) )
			$options = array('title'=>'Top Commentators', 'listDesc'=>'You commented; therefore you are loved:', 'excludeNames'=>'\'Lorna\', \'Administrator\'','limitList'=>'10', 'limitChar'=>'25', 'listNull'=>'Be the first to comment in my site.', 'filterUrl'=>'http://playboy.com', 'listType'=>'bul', 'listPeriod'=>'m', 'makeLink'=>'1', 'showInHome'=>'0');
		if ( $_POST['topcomm-submit'] ) {
			// Remember to sanitize and format use input appropriately.
			$options['title'] = $_POST['topcomm-title'];
			$options['excludeNames'] = $_POST['topcomm-excludeNames'];
			$options['limitList'] = $_POST['topcomm-limitList'];
			$options['limitChar'] = $_POST['topcomm-limitChar'];
			$options['listDesc'] = $_POST['topcomm-listDesc'];
			$options['listType'] = $_POST['topcomm-listType'];
			if($_POST['topcomm-listPeriodnum'] == '') {
				$options['listPeriod'] = $_POST['topcomm-listPeriod'];
			} else {
				$options['listPeriod'] = $_POST['topcomm-listPeriodnum'];
			}
			$options['listNull'] = $_POST['topcomm-listNull'];
			$options['filterUrl'] = $_POST['topcomm-filterUrl'];
  			$options['makeLink'] = $_POST['topcomm-makeLink'];
  			$options['showInHome'] = $_POST['topcomm-showInHome'];
  			update_option('widget_topcomm', $options);
  		}
   		// Be sure you format your options to be valid HTML attributes.
    		// Here is our little form segment. Notice that we don't need a
  		// complete form. This will be embedded into the existing form.
		$options = get_option('widget_topcomm');
		$title = htmlspecialchars(stripcslashes($options['title']), ENT_QUOTES);
		$excludeNames = htmlspecialchars(stripcslashes($options['excludeNames']), ENT_QUOTES);
		$limitList = $options['limitList'];
		$limitChar = $options['limitChar'];
		$listDesc = $options['listDesc'];
		$listType = $options['listType'];
		$listPeriod = $options['listPeriod'];
		$listNull = $options['listNull'];
		$filterUrl = $options['filterUrl'];
		$makeLink = $options['makeLink'];
		$showInHome = $options['showInHome'];
  		echo '<p style="text-align:right;"><label for="topcomm-title">' . ('Change widget title:') . ' <input style="width: 200px;" id="topcomm-title" name="topcomm-title" type="text" value="'.$title.'" /></label></p>';
  		echo '<p style="text-align:right;"><label for="topcomm-listDesc">' . ('Add description below the title:') . ' <input style="width: 200px;" id="topcomm-listDesc" name="topcomm-listDesc" type="text" value="'.$listDesc.'" /></label><br />Leave blank to exclude description.</p>';
  		echo '<p style="text-align:right;"><label for="topcomm-excludeNames">' . ('Exclude these users:') . ' <input style="width: 200px;" id="topcomm-excludeNames" name="topcomm-excludeNames" type="text" value="'.$excludeNames.'" /></label><br />Separate each name with a comma (,)</p>';
  		echo '<p style="text-align:right;"><label for="topcomm-listPeriod">' . ('Reset list every:') . '<select size="1" id="topcomm-listPeriod" name="topcomm-listPeriod"><option value="h"';
  		if($listPeriod== 'h')
  			echo ' selected';
  		echo '>Hour</option><option value="d"';
  		if($listPeriod== 'd')
  			echo ' selected';
  		echo '>Day</option><option value="w"';
  		if($listPeriod== 'w')
  			echo ' selected';
  		echo '>Week</option><option value="m"';
  		if($listPeriod== 'm' || is_null($listPeriod))
  			echo ' selected';
  		echo '>Month</option><option value="y"';
  		if($listPeriod== 'y')
  			echo ' selected';
  		echo '>Year</option><option value="y"';
  		if($listPeriod== 'a')
  			echo ' selected';
  		echo '>List all</option></select><br />Or specify number of days: <input style="width: 50px;" id="topcomm-listPeriodnum" name="topcomm-listPeriodnum" type="text" value="';
  		if (is_numeric($listPeriod))
   		 	echo $listPeriod;
  		echo '" /></label></p>';
  		echo '<p style="text-align:right;"><label for="topcomm-limitList">' . ('Limit number of names to:') . ' <input style="width: 200px;" id="topcomm-limitList" name="topcomm-limitList" type="text" value="'.$limitList.'" /></label><br />Enter numbers only</p>';
  		echo '<p style="text-align:right;"><label for="topcomm-limitChar">' . ('Limit characters in names to:') . ' <input style="width: 200px;" id="topcomm-limitChar" name="topcomm-limitChar" type="text" value="'.$limitChar.'" /></label><br />Enter numbers only</p>';
  		echo '<p style="text-align:right;"><label for="topcomm-listNull">' . ('Remarks for blank list:') . ' <input style="width: 200px;" id="topcomm-listNull" name="topcomm-listNull" type="text" value="'.$listNull.'" /></label></p>';
  		echo '<p style="text-align:right;"><label for="topcomm-filterUrl">' . ('Filter the following URLs:') . ' <input style="width: 200px;" id="topcomm-filterUrl" name="topcomm-filterUrl" type="text" value="'.$filterUrl.'" /></label><br />Separate each URl with a comma (,)</p>';
  		echo '<p style="text-align:right;"><label for="topcomm-listType">' . ('Display list type as:') . '<select size="1" id="topcomm-listType" name="topcomm-listType"><option value="bul"';
  		if($listType == 'bul')
  			echo ' selected';
  		echo '>Bulleted</option><option value="num"';
  		if($listType == 'num')
  			echo ' selected';
  		echo '>Numbered</option></select></label></p>';
  		echo '<p style="text-align:right;"><label for="topcomm-makeLink">' . ('Hyperlink each name?') . '<select size="1" id="topcomm-makeLink" name="topcomm-makeLink"><option value="1"';
  		if($makeLink == 1)
  			echo ' selected';
 		echo '>Yes</option><option value="0"';
  		if($makeLink == 0)
  			echo ' selected';
  		echo '>No</option></select></label></p>';
  		echo '<p style="text-align:right;"><label for="topcomm-showInHome">' . ('Show in home page only?') . '<select size="1" id="topcomm-showInHome" name="topcomm-showInHome"><option value="1"';
  		if($showInHome == 1)
  			echo ' selected';
 		echo '>Yes</option><option value="0"';
  		if($showInHome == 0)
  			echo ' selected';
  		echo '>No</option></select></label></p>';
  		echo '<input type="hidden" id="topcomm-submit" name="topcomm-submit" value="1" />';
  	}
	// This registers our widget so it appears with the other available
  	// widgets and can be dragged and dropped into any active sidebars.
  	register_sidebar_widget(array('Top Commentators', 'widgets'), 'widget_topcomm');

    	// This registers our optional widget control form. Because of this
  	// our widget will have a button that reveals a 300x100 pixel form.
  	register_widget_control(array('Top Commentators', 'widgets'), 'widget_topcomm_control', 410, 500);
}

// Run our code later in case this loads prior to any required plugins.
add_action('widgets_init', 'widget_topcomm_init');
?>